***********************************************; 
*                                             *;
* Paper 9341 SAS Global Forum 2016            *; 
* code modules                                *;
*                                             *;
***********************************************;


***********************************************;
* Please note ---                             *;
*                                             *;
*  Re-Start your SAS system at the start of   *;
*  running the code for each example. These   *;
*  code examples assume that the SAS system   *;
*  has just been restarted and none of the    *;
*  files marked as missing exist at the start *;
*  of the programs. running the code more     *;
*  than once during a session will change     *;
*  the messages output by SAS because the     *;
*  files or macro variables exist.            *;
*                                             *;
***********************************************;


***************************;
* Example 1               *;
***************************;                
 

DATA _null_;
   a = 5;
   CALL SYMPUTX('max_array_size',a,'G');
DATA temp;
   array second(&max_array_size) _temporary_ ;
   second(3)=15;
   DO I = 1 to &max_array_size;
	PUT second(I)=;
   END;
PROC PRINT data=temp;
RUN;


***************************;
* Example 2               *;
***************************;   
LIBNAME aa "c:\my_sas_dir";       /* The following NOTE appears when an input file does not exist*/

LIBNAME aa "c:\my_sas_code";       /* The following NOTE appears when an input file does exist   */

* Not all file actions produce notes;
FILENAME bb 'C:\My_Excel_Files';		    /* directory does exist     */

FILENAME cc 'C:\My_Excel_Files\shoes.csv';  /* file does exist         */
FILENAME dd 'c:\my_sas_dir';			   /* directory does not exist */


***************************;
* Example 3               *;
***************************;  

OPTION COMPRESS=binary;
DATA A(KEEP=i aa bb cc) B(KEEP=i d e f g); 
DO i = 1 TO 10; 
   aa = i*11;    bb = i*12;    cc = i*13; 	    OUTPUT A;
   aa = i*19;    bb = i*23;    cc = i*31; 	    OUTPUT A;
   IF MOD(i,3) = 0 THEN DO; 
         d = i+7;  e  = i*2;   f  = i**3; g = i*15; OUTPUT B;
         d = i+13; e = i*29;   f  = i**4; g = i*13; OUTPUT B;
   END;
   IF MOD(i,2) = 0 THEN DO;
         d = i+17;  e = i*42;  f  = i**5; g = i*25; OUTPUT B;
         d = i+53;  e = i*72;  f  = i**6; g = i*18; OUTPUT B;
   END;
END;
RUN;



***************************;
* Example 3  Sample 1     *;
***************************;  
DATA merged;
   MERGE a b;
   BY i;
RUN;


***************************;
* Example 3  Sample 2     *;
***************************; 
DATA merged;
   MERGE a b;
RUN;




***************************;
* Example 4               *;
***************************;
LIBNAME xx  "C:\my_sas_files';
  DATA test;
     SET xx.shoes;
     IF region  = 'Asia'    THEN a = 1;
     IF product = 'Boot'    THEN b = 2;
     q = 'help";
RUN;



***************************;
* Example 4  Corrected    *;
***************************;
LIBNAME xx  "C:\my_sas_files";
  DATA test;
     SET xx.shoes;
     IF region  = 'Asia'    THEN a = 1;
     IF product = 'Boot'    THEN b = 2;
     q = "help";
RUN;


***************************;
* Example 5   record size *;
***************************;
DATA a ;
a = "12345678901234567890";       output;
RUN;
DATA b ;
a = "12345678901234567890123456"; output;
a = "12345678901234567890765432"; output;
a = "abcdefghijklmnopqrstuvwxyz"; output;
RUN;
******************** Sample 1 *******************;
 DATA merged;
    MERGE a b;
    BY a;
RUN;
******************** Sample 2 *******************;
DATA merged;
    MERGE b a;
    BY a;
RUN;

***************************;
* Example 6 syntax error  *;
***************************;
DATA test;
   
   IF a=1 THEN b = c;
   IF a2= THEN d = e;
   q = "help";

RUN;


****************************************;
* Example 7 Cascading errors (Original)*;
****************************************;
DATA out_file_1; 
   SET sashelp.shoes (KEEP region sales WHERE(region=asia); 
RUN;

****************************************;
* Example 7 Cascading errors (Pass # 1)*;
****************************************;
DATA out_file_1; 
   SET sashelp.shoes (KEEP region sales WHERE(region=asia); 
RUN;

****************************************;
* Example 7 Cascading errors (Pass # 2)*;
****************************************;
DATA out_file_1; 
   SET sashelp.shoes (KEEP=region sales WHERE(region=asia); 
RUN;


****************************************;
* Example 7 Cascading errors (Pass # 3)*;
****************************************;
DATA out_file_1; 
   SET sashelp.shoes (KEEP=region sales WHERE=(region=asia);
RUN;


****************************************;
* Example 7 Cascading errors (Pass # 4)*;
****************************************;
DATA out_file_1; 
   SET sashelp.shoes (KEEP=region sales WHERE=(region=asia)); 
RUN;


****************************************;
* Example 7 Cascading errors (Pass # 5)*;
****************************************;
DATA out_file_1; 
   SET sashelp.shoes (KEEP=region sales WHERE=(region="asia")); 
RUN;


****************************************;
* Example 7 Cascading errors (Pass # 6)*;
****************************************;
DATA out_file_1; 
   SET sashelp.shoes (KEEP=region sales WHERE=(UPCASE(region)="ASIA")); 
RUN;



***************************;
* Example 8 Run-Time Error*;
***************************;
DATA test;
     SET sashelp.shoes;
     IF z=1 THEN b = c;
     IF z=2 THEN r = s;
     q = "help";
RUN;



***************************;
* Example 9 Logic Errors  *;
***************************;
DATA test/debug;
   a = 1;     b = 2;
   LINK add_a_AND_b;
   IF c > 10 THEN GOTO exit;
return;
exit:
add_a_and_b:
   c = SUM(a,b);
   LINK exit;
RETURN;
RUN;

***************************;
* Example 9 correct Logic *;
***************************;
DATA test/debug;
   a = 1;     b = 2;
   LINK add_a_AND_b;
   IF c > 10 THEN GOTO exit;
return;
exit:
add_a_and_b:
   c = SUM(a,b);
*   LINK exit;
RETURN;
RUN;


***************************;
* Example 10  Macro Error *;
***************************;
DATA temp;
   a = 5;
   CALL SYMPUTX('max_array_size1',a,'G');
   ARRAY second(&max_array_size1) _TEMPORARY_ ;
   second(3)=15;
   DO I = 1 TO &max_array_size1;
	PUT second(I)=;
   END;
RUN;


*************************************;
* Example 10  Macro Error corrected *;
*************************************;
DATA _null_;
   a = 5;
   CALL SYMPUTX('max_array_size1',a,'G');
RUN;
DATA temp;
   ARRAY second(&max_array_size1) _TEMPORARY_ ;
   second(3)=15;
   DO I = 1 TO &max_array_size1;
	PUT second(I)=;
   END;
RUN;



**************************************************;
* Example 11  Macro Syntax Error (original code) *;
**************************************************;
%LET path = "C:\temp\";
%LET dept = "my_files""; 
%MACRO test_macro;
   LIBNAME xx = "&path.&dept.";
   DATA errors; 
      SET xx.shoes;
   RUN; 
%MEND test_macro; 
%test_macro;
RUN;


**************************************************;
* Example 11  Macro Syntax Error (for Log # 18)  *;
* NOTE -- Macro vars path and dept changed to    *;
*         produce the error in output log 18     *;
* NOTE -- The log in the paper is not changed    *;
**************************************************;
%LET path4 = "C:\temp\";
%LET dept4 = "my_files"";
%MACRO test_macro;
   LIBNAME xx = "&path4.&dept4.";
   DATA errors;
      SET xx.shoes;
   RUN;
%MEND test_macro;
%test_macro;
RUN;

**************************************************;
* Example 11  Macro Syntax Error (for Log # 19)  *;
* NOTE -- Macro vars path and dept changed to    *;
*         produce the error in output log 19     *;
* NOTE -- The log in the paper is not changed    *;
**************************************************;
%LET path5 = "C:\temp\";
%LET dept5 = "my_files";
%MACRO test_macro;
   LIBNAME xx = "&path5.&dept5.";
   DATA errors;
      SET xx.shoes;
   RUN;
%MEND test_macro;
%test_macro;
RUN;


**************************************************;
* Example 11  Macro Syntax Error (for Log # 20)  *;
* NOTE -- Macro vars path and dept changed to    *;
*         produce the error in output log 20     *;
* NOTE -- The log in the paper is not changed    *;
* NOTE -- The SAS file "C:\temp\my_iles\SHOES"    *;
*         must exist for for this to work.       *;
**************************************************;
%LET path6 = C:\temp\;
%LET dept6 = my_files;
%MACRO test_macro;
   LIBNAME xx "&path6.&dept6.";
   DATA errors;
      SET xx.shoes;
   RUN;
%MEND test_macro;
%test_macro;


**************************************************;
* Example 12  Suppress all NOTE messages         *;
**************************************************;
OPTIONS nonotes;
%LET path = C:\temp\;
%LET dept = my_files;
%MACRO test_macro;
   LIBNAME xx "&path.&dept.";
   DATA errors;
      SET xx.shoes;
   RUN;
%MEND test_macro;
 %test_macro;
 RUN;


**************************************************;
* Example 14  SUPPRESS DATA CONVERSION ERRORS    *;
*             WHEN USING INPUT STATEMENTS        *;
**************************************************;
DATA test; 
INFILE CARDS;
INPUT @1 my_number ?? 8.2; 
CARDS; 
not_numb
12345.67
;;;;
RUN;

**************************************************;
* Example 15  SUPPRESS MESSAGES WHEN DROPPING,   *; 
*      KEEPING, OR RENAMING SAS VARIABLES        *;
*      Output Log # 23                           *;
**************************************************;
OPTION DKRICOND=ERROR DKROCOND=ERROR;
  DATA shoes(rename=(smith=jones));
     SET sashelp.shoes(keep=smith);
  RUN;

**************************************************;
* Example 15  SUPPRESS MESSAGES WHEN DROPPING,   *; 
*      KEEPING, OR RENAMING SAS VARIABLES        *;
*      Output Log # 24                           *;
**************************************************;
OPTION DKRICOND =NOWARN DKROCOND=NOWARNING;
DATA shoes(rename=(smith=jones));
   SET sashelp.shoes(keep=smith);
RUN;


**************************************************;
* Example 16  MAKE UP YOUR OWN NOTE, WARNING,    *;
*      AND ERROR MESSAGES                        *;
*      Output Log # 25                           *;
**************************************************;
DATA _null_;
   PUT 'NOTE       my note       1 - shows up as text on the log';
   PUT 'WARNING    my warning    1 - shows up as text on the log';
   PUT 'ERROR      my error      1 - shows up as text on the log';
RUN;

DATA _null_;
   PUT 'NOTE:      my note       1 - shows up as a NOTE on the log';
   PUT 'WARNING:   my warning    1 - shows up as a WARNING on the log';
   PUT 'ERROR:     my error      1 - shows up as a ERROR on the log';
RUN;

**************************************************;
* Example 17  SHOW MORE INFORMATION ABOUT THE    *; 
*      RUNTIME ENVIRONMENT                       *;
*      Output Log # 26                           *;
**************************************************;
OPTION FULLSTIMER;
DATA shoes;
   SET sashelp.shoes;
RUN;






